@@ -1,13 +1,24 @@
+import hashlib
+import json
 from logging import getLogger
 
 from django.conf import settings
 from django.db import models
 from django.db.models import UniqueConstraint, Q
+from django.db.utils import IntegrityError
 from django.contrib.postgres import fields as psql_fields
 from django.contrib.postgres import search as psql_search
-from django_lifecycle import AFTER_UPDATE, BEFORE_UPDATE, hook
+from django_lifecycle import (
+    AFTER_UPDATE,
+    AFTER_DELETE,
+    AFTER_CREATE,
+    BEFORE_UPDATE,
+    BEFORE_SAVE,
+    hook,
+)
 
 from pulpcore.plugin.models import (
+    AutoAddObjPermsMixin,
     BaseModel,
     Content,
     Remote,
@@ -18,6 +29,8 @@ from pulpcore.plugin.models import (
     Task,
     EncryptedTextField,
 )
+from pulpcore.plugin.repo_version_utils import remove_duplicates, validate_repo_version
+
 from .downloaders import AnsibleDownloaderFactory
 
 
@@ -100,6 +113,14 @@ class Tag(BaseModel):
         return self.name
 
 
+class AnsibleNamespace(BaseModel):
+    """
+    A model representing a Namespace. This should be used for permissions.
+    """
+
+    name = models.CharField(max_length=64, unique=True, editable=True)
+
+
 class CollectionVersion(Content):
     """
     A content type representing a CollectionVersion.
@@ -200,6 +221,28 @@ class CollectionVersion(Content):
         ]
 
 
+class CollectionVersionMark(Content):
+    """
+    A content type representing a mark that is attached to a content unit.
+
+    Fields:
+        value (models.CharField): The value of the mark.
+        marked_collection (models.ForeignKey): Reference to a CollectionVersion.
+    """
+
+    PROTECTED_FROM_RECLAIM = False
+    TYPE = "collection_mark"
+
+    value = models.SlugField()
+    marked_collection = models.ForeignKey(
+        CollectionVersion, null=False, on_delete=models.CASCADE, related_name="marks"
+    )
+
+    class Meta:
+        default_related_name = "%(app_label)s_%(model_name)s"
+        unique_together = ("value", "marked_collection")
+
+
 class CollectionVersionSignature(Content):
     """
     A content type representing a signature that is attached to a content unit.
@@ -232,6 +275,78 @@ class CollectionVersionSignature(Content):
         unique_together = ("pubkey_fingerprint", "signed_collection")
 
 
+class AnsibleNamespaceMetadata(Content):
+    """
+    A content type representing the Namespace metadata of a Collection.
+
+    Namespace metadata can have an avatar which is stored as an associated artifact upon upload.
+
+    Fields:
+        name (models.CharField): The required name of the Namespace
+        company (models.CharField): Optional namespace owner company name.
+        email (models.CharField): Optional namespace contact email.
+        description (models.CharField): Namespace brief description.
+        resources (models.TextField): Namespace resources page in markdown format.
+        links (psql_fields.HStore): Labeled related links.
+        avatar_sha256 (models.CharField): SHA256 digest of avatar image.
+        metadata_sha256(models.CharField): SHA256 digest of all other metadata fields.
+
+    Relations:
+        namespace (AnsibleNamespace): A link to the Namespace model used for permissions.
+    """
+
+    TYPE = "namespace"
+    repo_key_fields = ("name",)
+    # fields on the existing model in galaxy_ng
+    name = models.CharField(max_length=64, blank=False)
+    company = models.CharField(max_length=64, blank=True, default="")
+    email = models.CharField(max_length=256, blank=True, default="")
+    description = models.CharField(max_length=256, blank=True, default="")
+    resources = models.TextField(blank=True, default="")
+
+    links = psql_fields.HStoreField(default=dict)
+    avatar_sha256 = models.CharField(max_length=64, null=True)
+
+    # Hash of the values of all the fields mentioned above.
+    # Content uniqueness constraint.
+    metadata_sha256 = models.CharField(max_length=64, db_index=True, blank=False)
+    namespace = models.ForeignKey(
+        AnsibleNamespace, on_delete=models.PROTECT, related_name="metadatas"
+    )
+
+    @property
+    def avatar_artifact(self):
+        if self.avatar_sha256:
+            return self._artifacts.get(sha256=self.avatar_sha256)
+
+        return None
+
+    @hook(BEFORE_SAVE)
+    def calculate_metadata_sha256(self):
+        """Calculates the metadata_sha256 from the other metadata fields."""
+        metadata = {
+            "name": self.name,
+            "company": self.company,
+            "email": self.email,
+            "description": self.description,
+            "resources": self.resources,
+            "links": self.links,
+            "avatar_sha256": self.avatar_sha256,
+        }
+
+        metadata_json = json.dumps(metadata, sort_keys=True).encode("utf-8")
+        hasher = hashlib.sha256(metadata_json)
+        if self.metadata_sha256:
+            # If we are creating from sync, assert that calculated hash == synced hash
+            if self.metadata_sha256 != hasher.hexdigest():
+                raise IntegrityError("Calculated digest does not equal passed in digest")
+        self.metadata_sha256 = hasher.hexdigest()
+
+    class Meta:
+        default_related_name = "%(app_label)s_%(model_name)s"
+        unique_together = ("metadata_sha256",)
+
+
 class DownloadLog(BaseModel):
     """
     A download log for content units by user, IP and org_id.
@@ -257,7 +372,7 @@ class DownloadLog(BaseModel):
     )
 
 
-class RoleRemote(Remote):
+class RoleRemote(Remote, AutoAddObjPermsMixin):
     """
     A Remote for Ansible content.
     """
@@ -266,9 +381,15 @@ class RoleRemote(Remote):
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = [("manage_roles_roleremote", "Can manage roles on role remotes")]
 
 
-class CollectionRemote(Remote):
+def _get_last_sync_task(pk):
+    sync_tasks = Task.objects.filter(name__contains="sync", reserved_resources_record__icontains=pk)
+    return sync_tasks.order_by("-pulp_created").first()
+
+
+class CollectionRemote(Remote, AutoAddObjPermsMixin):
     """
     A Remote for Collection content.
     """
@@ -302,6 +423,10 @@ class CollectionRemote(Remote):
             self._download_factory = AnsibleDownloaderFactory(self)
             return self._download_factory
 
+    @property
+    def last_sync_task(self):
+        return _get_last_sync_task(self.pk)
+
     @hook(
         AFTER_UPDATE,
         when_any=["url", "requirements_file", "sync_dependencies", "signed_only"],
@@ -314,9 +439,10 @@ class CollectionRemote(Remote):
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = (("manage_roles_collectionremote", "Can manage roles on collection remotes"),)
 
 
-class GitRemote(Remote):
+class GitRemote(Remote, AutoAddObjPermsMixin):
     """
     A Remote for Collection content hosted in Git repositories.
     """
@@ -328,6 +454,9 @@ class GitRemote(Remote):
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = [
+            ("manage_roles_gitremote", "Can manage roles on git remotes"),
+        ]
 
 
 class AnsibleCollectionDeprecated(Content):
@@ -345,13 +474,14 @@ class AnsibleCollectionDeprecated(Content):
         unique_together = ("namespace", "name")
 
 
-class AnsibleRepository(Repository):
+class AnsibleRepository(Repository, AutoAddObjPermsMixin):
     """
     Repository for "ansible" content.
 
     Fields:
 
         last_synced_metadata_time (models.DateTimeField): Last synced metadata time.
+        private (models.BooleanField): Indicator if this repository is private
     """
 
     TYPE = "ansible"
@@ -360,16 +490,30 @@ class AnsibleRepository(Repository):
         CollectionVersion,
         AnsibleCollectionDeprecated,
         CollectionVersionSignature,
+        AnsibleNamespaceMetadata,
+        CollectionVersionMark,
     ]
-    REMOTE_TYPES = [RoleRemote, CollectionRemote]
+    REMOTE_TYPES = [RoleRemote, CollectionRemote, GitRemote]
 
     last_synced_metadata_time = models.DateTimeField(null=True)
     gpgkey = models.TextField(null=True)
+    private = models.BooleanField(default=False)
+
+    @property
+    def last_sync_task(self):
+        return _get_last_sync_task(self.pk)
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
 
-        permissions = (("modify_ansible_repo_content", "Can modify ansible repository content"),)
+        permissions = [
+            ("rebuild_metadata_ansiblerepository", "Can rebuild metadata on the repository"),
+            ("repair_ansiblerepository", "Can repair the repository"),
+            ("sign_ansiblerepository", "Can sign content on the repository"),
+            ("sync_ansiblerepository", "Can start a sync task on the repository"),
+            ("manage_roles_ansiblerepository", "Can manage roles on repositories"),
+            ("modify_ansible_repo_content", "Can modify repository content"),
+        ]
 
     def finalize_new_version(self, new_version):
         """Finalize repo version."""
@@ -377,8 +521,8 @@ class AnsibleRepository(Repository):
             base_version=new_version.base_version
         ).filter(pulp_type=CollectionVersion.get_pulp_type())
 
-        # Remove any deprecated and signature content associated with the removed collection
-        # versions
+        # Remove any deprecated, signature, mark and namespace content
+        # associated with the removed collection versions
         for version in removed_collection_versions:
             version = version.cast()
 
@@ -387,28 +531,43 @@ class AnsibleRepository(Repository):
             )
             new_version.remove_content(signatures)
 
+            marks = new_version.get_content(
+                content_qs=CollectionVersionMark.objects.filter(marked_collection=version)
+            )
+            new_version.remove_content(marks)
+
             other_collection_versions = new_version.get_content(
                 content_qs=CollectionVersion.objects.filter(collection=version.collection)
             )
 
-            # AnsibleCollectionDeprecated applies to all collection versions in a repository,
-            # so only remove it if there are no more collection versions for the specified
-            # collection present.
+            # AnsibleCollectionDeprecated and Namespace applies to all collection versions in a
+            # repository, so only remove it if there are no more collection versions for the
+            # specified collection present.
             if not other_collection_versions.exists():
                 deprecations = new_version.get_content(
                     content_qs=AnsibleCollectionDeprecated.objects.filter(
                         namespace=version.namespace, name=version.name
                     )
                 )
+                namespace = new_version.get_content(
+                    content_qs=AnsibleNamespaceMetadata.objects.filter(name=version.namespace)
+                )
 
                 new_version.remove_content(deprecations)
+                new_version.remove_content(namespace)
+        remove_duplicates(new_version)
+        validate_repo_version(new_version)
+
+        from pulp_ansible.app.tasks.collectionversion_index import update_index
+
+        update_index(repository_version=new_version, is_latest=True)
 
     @hook(BEFORE_UPDATE, when="remote", has_changed=True)
     def _reset_repository_last_synced_metadata_time(self):
         self.last_synced_metadata_time = None
 
 
-class AnsibleDistribution(Distribution):
+class AnsibleDistribution(Distribution, AutoAddObjPermsMixin):
     """
     A Distribution for Ansible content.
     """
@@ -417,3 +576,33 @@ class AnsibleDistribution(Distribution):
 
     class Meta:
         default_related_name = "%(app_label)s_%(model_name)s"
+        permissions = [("manage_roles_ansibledistribution", "Can manage roles on distributions")]
+
+    @hook(AFTER_CREATE)
+    @hook(AFTER_UPDATE)
+    @hook(AFTER_DELETE)
+    def _update_index(self):
+        from pulp_ansible.app.tasks.collectionversion_index import update_distribution_index
+
+        update_distribution_index(self)
+
+
+class CrossRepositoryCollectionVersionIndex(models.Model):
+    """
+    A model that indexes all CV content across all repositories.
+    """
+
+    repository = models.ForeignKey(AnsibleRepository, on_delete=models.CASCADE)
+    repository_version = models.ForeignKey(RepositoryVersion, on_delete=models.CASCADE, null=True)
+    collection_version = models.ForeignKey(CollectionVersion, on_delete=models.CASCADE)
+    namespace_metadata = models.ForeignKey(
+        AnsibleNamespaceMetadata, null=True, on_delete=models.SET_NULL
+    )
+
+    is_deprecated = models.BooleanField()
+    is_signed = models.BooleanField()
+    is_highest = models.BooleanField()
+
+    class Meta:
+        default_related_name = "%(app_label)s_%(model_name)s"
+        unique_together = ("repository", "repository_version", "collection_version")
