@@ -1,14 +1,20 @@
 from gettext import gettext as _
 
+import json
+
+from django.db import transaction
 from django.conf import settings
 from jsonschema import Draft7Validator
 from rest_framework import serializers
+from drf_spectacular.utils import extend_schema_field
 
-from pulpcore.plugin.models import Artifact, SigningService
+from galaxy_importer.constants import NAME_REGEXP
+from pulpcore.plugin.models import Artifact, ContentArtifact, SigningService
 from pulpcore.plugin.serializers import (
     DetailRelatedField,
     ContentChecksumSerializer,
     ModelSerializer,
+    NoArtifactContentSerializer,
     NoArtifactContentUploadSerializer,
     RelatedField,
     RemoteSerializer,
@@ -19,13 +25,18 @@ from pulpcore.plugin.serializers import (
     DistributionSerializer,
     RepositoryVersionRelatedField,
     validate_unknown_fields,
+    TaskSerializer,
 )
+from pulpcore.plugin.util import get_url
 from rest_framework.exceptions import ValidationError
 
 from .models import (
     AnsibleDistribution,
+    CollectionVersionMark,
     GitRemote,
     RoleRemote,
+    AnsibleNamespace,
+    AnsibleNamespaceMetadata,
     AnsibleRepository,
     Collection,
     CollectionImport,
@@ -126,6 +137,7 @@ class AnsibleRepositorySerializer(RepositorySerializer):
     last_synced_metadata_time = serializers.DateTimeField(
         help_text=_("Last synced metadata time."), allow_null=True, required=False
     )
+    last_sync_task = TaskSerializer(read_only=True)
     gpgkey = serializers.CharField(
         help_text="Gpg public key to verify collection signatures against",
         required=False,
@@ -136,6 +148,8 @@ class AnsibleRepositorySerializer(RepositorySerializer):
         fields = RepositorySerializer.Meta.fields + (
             "last_synced_metadata_time",
             "gpgkey",
+            "last_sync_task",
+            "private",
         )
         model = AnsibleRepository
 
@@ -211,6 +225,8 @@ class CollectionRemoteSerializer(RemoteSerializer):
         default=False,
     )
 
+    last_sync_task = TaskSerializer(read_only=True)
+
     def validate(self, data):
         """
         Validate collection remote data.
@@ -262,6 +278,7 @@ class CollectionRemoteSerializer(RemoteSerializer):
             "token",
             "sync_dependencies",
             "signed_only",
+            "last_sync_task",
         )
         model = CollectionRemote
 
@@ -735,6 +752,207 @@ class CollectionVersionSignatureSerializer(NoArtifactContentUploadSerializer):
         )
 
 
+class NamespaceLinkSerializer(serializers.Serializer):
+    """
+    Provides backwards compatible interface for links with the legacy
+    GalaxyNG API.
+    """
+
+    url = serializers.URLField(max_length=256, allow_blank=False)
+    name = serializers.CharField(max_length=256, allow_blank=False)
+
+
+@extend_schema_field(NamespaceLinkSerializer(many=True))
+class NamespaceLinkField(serializers.HStoreField):
+    """
+    Provides backwards compatible interface for links with the legacy
+    GalaxyNG API.
+    """
+
+    def get_value(self, dictionary):
+        data = dictionary.get(self.field_name, [])
+
+        # The open api client sends data as a form request rather than json
+        # because of the avatar URL. It converts a list like
+        # [{"foo": "bar"}, {"bar": "foo"}] into "{'foo': 'bar'}, {'bar': 'foo'}"
+        # This is a best effort attempt to capture that data and convert it into
+        # valid JSON and then transform it into a list that the API can understand
+        if isinstance(data, str):
+            try:
+                data = f"[{data}]".replace("'", '"')
+                return json.loads(data)
+            except json.decoder.JSONDecodeError:
+                raise ValidationError(detail={"links": "Must be valid JSON"})
+
+        return super().get_value(dictionary)
+
+    def to_internal_value(self, data):
+        if isinstance(data, dict):
+            transformed = data
+        else:
+            transformed = {x["name"]: x["url"] for x in data}
+        return super().to_internal_value(transformed)
+
+    def to_representation(self, value):
+        return [{"name": x, "url": value[x]} for x in value]
+
+
+class AnsibleNamespaceMetadataSerializer(NoArtifactContentSerializer):
+    """
+    A serializer for Namespaces.
+    """
+
+    name = serializers.RegexField(
+        NAME_REGEXP,
+        min_length=3,
+        max_length=64,
+        allow_blank=False,
+        help_text=_("Required named, only accepts lowercase, numbers and underscores."),
+    )
+    company = serializers.CharField(
+        max_length=64,
+        allow_blank=True,
+        required=False,
+        help_text=_("Optional namespace company owner."),
+    )
+    email = serializers.CharField(
+        max_length=256,
+        allow_blank=True,
+        required=False,
+        help_text=_("Optional namespace contact email."),
+    )
+    description = serializers.CharField(
+        max_length=256, allow_blank=True, required=False, help_text=_("Optional short description.")
+    )
+    resources = serializers.CharField(
+        allow_blank=True, required=False, help_text=_("Optional resource page in markdown format.")
+    )
+    links = NamespaceLinkField(
+        child=serializers.URLField(max_length=256),
+        required=False,
+        help_text=_("Labeled related links."),
+    )
+    avatar = serializers.ImageField(
+        write_only=True, required=False, help_text=_("Optional avatar image for Namespace")
+    )
+    avatar_sha256 = serializers.CharField(
+        max_length=64, read_only=True, help_text=_("SHA256 digest of avatar image if present.")
+    )
+    avatar_url = serializers.SerializerMethodField(
+        help_text=_("Download link for avatar image if present.")
+    )
+    metadata_sha256 = serializers.CharField(read_only=True)
+
+    def get_avatar_url(self, obj):
+        """Return the avatar url"""
+        if obj.avatar_sha256:
+            return settings.ANSIBLE_API_HOSTNAME + get_url(obj) + "avatar/"
+        else:
+            return None
+
+    def validate(self, data):
+        """Check that avatar_sha256 is set if avatar was present in upload."""
+        if self.instance:
+            if (name := data.get("name", None)) and name != self.instance.name:
+                raise serializers.ValidationError(_("Name can not be changed in an update"))
+
+        if "artifact" in self.context:
+            if "avatar_sh256" not in data:
+                avatar_artifact = Artifact.objects.get(pk=self.context["artifact"])
+                data["avatar_sha256"] = avatar_artifact.sha256
+
+        return super().validate(data)
+
+    @transaction.atomic
+    def create(self, validated_data):
+        """Create the Namespace and add it to the Repository if present."""
+        namespace, created = AnsibleNamespace.objects.get_or_create(name=validated_data["name"])
+        metadata = AnsibleNamespaceMetadata(namespace=namespace, **validated_data)
+        metadata.calculate_metadata_sha256()
+        content = AnsibleNamespaceMetadata.objects.filter(
+            metadata_sha256=metadata.metadata_sha256
+        ).first()
+        if content:
+            content.touch()
+        else:
+            metadata.save()
+            content = metadata
+            if metadata.avatar_sha256:
+                ContentArtifact.objects.create(
+                    artifact_id=self.context["artifact"],
+                    content=content,
+                    relative_path=f"{metadata.name}-avatar",
+                )
+
+        repository = self.context.pop("repository", None)
+        if repository:
+            repository = AnsibleRepository.objects.get(pk=repository)
+            content_to_add = AnsibleNamespaceMetadata.objects.filter(pk=content.pk)
+
+            with repository.new_version() as new_version:
+                new_version.add_content(content_to_add)
+        return content
+
+    class Meta:
+        model = AnsibleNamespaceMetadata
+        fields = (
+            "pulp_href",
+            "name",
+            "company",
+            "email",
+            "description",
+            "resources",
+            "links",
+            "avatar",
+            "avatar_sha256",
+            "avatar_url",
+            "metadata_sha256",
+        )
+
+
+class CollectionVersionMarkSerializer(NoArtifactContentSerializer):
+    """
+    A serializer for mark models.
+    """
+
+    marked_collection = DetailRelatedField(
+        help_text=_("The content this mark is pointing to."),
+        view_name_pattern=r"content(-.*/.*)-detail",
+        queryset=CollectionVersion.objects.all(),
+    )
+    value = serializers.SlugField(
+        help_text=_("The string value of this mark."),
+        allow_null=False,
+    )
+
+    class Meta:
+        model = CollectionVersionMark
+        fields = ["pulp_created", "pulp_href", "marked_collection", "value"]
+
+
+class AnsibleRepositoryMarkSerializer(serializers.Serializer):
+    """
+    A serializer for the mark action.
+    """
+
+    content_units = serializers.ListField(
+        required=True,
+        help_text=_(
+            "List of collection version hrefs to mark, use * to mark all content in repository"
+        ),
+    )
+    value = serializers.SlugField(
+        required=True,
+        help_text=_("The string value of this mark."),
+    )
+
+    def validate_content_units(self, value):
+        """Make sure the list is correctly formatted."""
+        if len(value) > 1 and "*" in value:
+            raise serializers.ValidationError("Cannot supply content units and '*'.")
+        return value
+
+
 class AnsibleRepositorySignatureSerializer(serializers.Serializer):
     """
     A serializer for the signing action.
@@ -789,6 +1007,40 @@ class CollectionImportDetailSerializer(CollectionImportListSerializer):
         fields = CollectionImportListSerializer.Meta.fields + ("error", "messages")
 
 
+class CollectionVersionCopyMoveSerializer(serializers.Serializer):
+    """
+    Copy or move collections from a source repository into one or more destinations.
+
+    This will carry associated content like Signatures and Marks along.
+    """
+
+    collection_versions = DetailRelatedField(
+        required=True,
+        view_name=r"content-ansible/collection_versions-detail",
+        queryset=CollectionVersion.objects.all(),
+        help_text=_("A list of collection versions to move or copy."),
+        many=True,
+    )
+
+    destination_repositories = DetailRelatedField(
+        required=True,
+        view_name="repositories-ansible/ansible-detail",
+        queryset=AnsibleRepository.objects.all(),
+        help_text=_("List of repository HREFs to put content in."),
+        many=True,
+    )
+
+    signing_service = RelatedField(
+        required=False,
+        view_name="signing-services-detail",
+        queryset=SigningService.objects.all(),
+        help_text=_(
+            "HREF for a signing service. This will be used to sign the collection "
+            "before moving putting it in any new repositories."
+        ),
+    )
+
+
 class CopySerializer(serializers.Serializer):
     """
     A serializer for Content Copy API.
