@@ -55,11 +55,14 @@ from semantic_version.base import Always
 from pulp_ansible.app.constants import PAGE_SIZE
 from pulp_ansible.app.models import (
     AnsibleCollectionDeprecated,
+    AnsibleNamespace,
+    AnsibleNamespaceMetadata,
     AnsibleRepository,
     Collection,
     CollectionImport,
     CollectionRemote,
     CollectionVersion,
+    CollectionVersionMark,
     CollectionVersionSignature,
     Tag,
 )
@@ -418,7 +421,7 @@ def _get_backend_storage_url(artifact_file):
     return url
 
 
-def _update_highest_version(collection_version):
+def _update_highest_version(collection_version, save=False):
     """
     Checks if this version is greater than the most highest one.
 
@@ -427,19 +430,44 @@ def _update_highest_version(collection_version):
     equals False on the last highest version and True on this version.
     Otherwise does nothing.
     """
-    last_highest = collection_version.collection.versions.filter(is_highest=True).first()
-    if not last_highest:
-        collection_version.is_highest = True
-        return None
-    if Version(collection_version.version) > Version(last_highest.version):
-        last_highest.is_highest = False
-        collection_version.is_highest = True
-        last_highest.save()
-        collection_version.save()
 
-    elif collection_version.is_highest and collection_version.version != last_highest.version:
-        collection_version.is_highest = False
-        collection_version.save()
+    def is_new_highest(new, old):
+        if bool(new.prerelease) == bool(old.prerelease):
+            return new > old
+        return bool(old.prerelease)
+
+    # did we have one set previously for this collection?
+    last_highest = collection_version.collection.versions.filter(is_highest=True).first()
+
+    if not last_highest:
+        # we only have one version, so mark it as the highest
+        if collection_version.collection.versions.count() == 1:
+            collection_version.is_highest = True
+            if save:
+                collection_version.save(update_fields=["is_highest"])
+            return
+
+        # compute highest from the whole list ...
+        highest = None
+        for cv in collection_version.collection.versions.all():
+            sv = Version(cv.version)
+            if highest is None or is_new_highest(sv, highest[0]):
+                highest = (sv, cv)
+
+        highest[1].is_highest = True
+        if highest[1] != collection_version or save:
+            highest[1].save()
+        return
+
+    # exit if the new CV is not higher
+    if not is_new_highest(Version(collection_version.version), Version(last_highest.version)):
+        return
+
+    last_highest.is_highest = False
+    last_highest.save(update_fields=["is_highest"])
+    collection_version.is_highest = True
+    if save:
+        collection_version.save(update_fields=["is_highest"])
 
 
 class AnsibleDeclarativeVersion(DeclarativeVersion):
@@ -507,6 +535,8 @@ class CollectionSyncFirstStage(Stage):
         self._unpaginated_collection_version_metadata = None
         self.optimize = optimize
         self.last_synced_metadata_time = None
+        self.namespaces_seen = set()
+        self._unpaginated_namespace_metadata = None
 
         # Interpret download policy
         self.deferred_download = self.remote.policy != Remote.IMMEDIATE
@@ -577,14 +607,15 @@ class CollectionSyncFirstStage(Stage):
         )
         cv_unique = attrgetter("namespace", "name", "version")(collection_version)
         fullname, version = f"{cv_unique[0]}.{cv_unique[1]}", cv_unique[2]
-        if fullname in self.exclude_info and version in self.exclude_info[fullname]:
+        if fullname in self.exclude_info and Version(version) in self.exclude_info[fullname]:
             return
         if cv_unique in self.already_synced:
             return
         self.already_synced.add(cv_unique)
 
         info = metadata["metadata"]
-        signatures = metadata.get("signatures")
+        signatures = metadata.get("signatures", [])
+        marks = metadata.get("marks", [])
 
         if self.signed_only and not signatures:
             return
@@ -629,7 +660,7 @@ class CollectionSyncFirstStage(Stage):
         await self.parsing_metadata_progress_bar.aincrement()
         await self.put(d_content)
 
-        if signatures:
+        if signatures or marks:
             collection_version = await d_content.resolution()
             for signature in signatures:
                 sig = signature["signature"]
@@ -641,6 +672,79 @@ class CollectionSyncFirstStage(Stage):
                 )
                 await self.put(DeclarativeContent(content=cv_signature))
 
+            for mark_value in marks:
+                cv_mark = CollectionVersionMark(
+                    marked_collection=collection_version,
+                    value=mark_value,
+                )
+                await self.put(DeclarativeContent(content=cv_mark))
+
+        # Process syncing CV Namespace Metadata if present
+        namespace_sha = metadata["namespace"].get("metadata_sha256")
+        if namespace_sha:
+            namespace = collection_version.namespace
+
+            if namespace not in self.namespaces_seen:
+                if await self._add_namespace(namespace, namespace_sha):
+                    self.namespaces_seen.add(namespace)
+
+    async def _add_namespace(self, name, namespace_sha):
+        """Adds A Namespace metadata content to the pipeline."""
+
+        try:
+            ns = await sync_to_async(AnsibleNamespaceMetadata.objects.get)(
+                metadata_sha256=namespace_sha
+            )
+            await self.put(DeclarativeContent(ns))
+            return True
+        except AnsibleNamespaceMetadata.DoesNotExist:
+            pass
+
+        endpoint, api_version = await self._get_root_api(self.remote.url)
+        namespace_url = f"{endpoint}/namespaces/{name}"
+        downloader = self.remote.get_downloader(
+            url=namespace_url, silence_errors_for_response_codes={404}
+        )
+        try:
+            result = await downloader.run()
+        except FileNotFoundError:
+            pass
+        else:
+            namespace = parse_metadata(result)
+            links = namespace.get("links", None)
+
+            # clean up the galaxy API for pulp
+            if links:
+                namespace["links"] = {x["name"]: x["url"] for x in links}
+            else:
+                namespace["links"] = dict()
+
+            for key in ("pulp_href", "groups", "id", "related_fields"):
+                namespace.pop(key, None)
+
+            url = namespace.pop("avatar_url", None)
+
+            da = (
+                [
+                    DeclarativeArtifact(
+                        Artifact(sha256=namespace["avatar_sha256"]),
+                        url=url,
+                        remote=self.remote,
+                        relative_path=f"{name}-avatar",
+                        deferred_download=False,
+                    )
+                ]
+                if url
+                else None
+            )
+
+            namespace = AnsibleNamespaceMetadata(**namespace)
+            dc = DeclarativeContent(namespace, d_artifacts=da)
+            await self.put(dc)
+            return True
+
+        return False
+
     async def _add_collection_version_from_git(self, url, gitref, metadata_only):
         d_content = await declarative_content_from_git_repo(
             self.remote, url, gitref, metadata_only=False
@@ -700,6 +804,8 @@ class CollectionSyncFirstStage(Stage):
                 break
             page_num = page_num + 1
 
+        self.parsing_metadata_progress_bar.total += len(tasks)
+        await self.parsing_metadata_progress_bar.asave(update_fields=["total"])
         await asyncio.gather(*tasks)
 
     async def _read_from_downloaded_metadata(self, name, namespace, requirement):
@@ -745,6 +851,8 @@ class CollectionSyncFirstStage(Stage):
                             )
                         )
                     )
+        self.parsing_metadata_progress_bar.total += len(tasks)
+        await self.parsing_metadata_progress_bar.asave(update_fields=["total"])
         await asyncio.gather(*tasks)
 
     async def _fetch_collection_metadata(self, requirements_entry):
@@ -856,6 +964,8 @@ class CollectionSyncFirstStage(Stage):
                         )
                     )
 
+        self.parsing_metadata_progress_bar.total = len(tasks)
+        await self.parsing_metadata_progress_bar.asave(update_fields=["total"])
         await asyncio.gather(*tasks)
 
     async def _find_all_collections(self):
@@ -897,6 +1007,8 @@ class CollectionSyncFirstStage(Stage):
                 break
             page_num = page_num + 1
 
+        self.parsing_metadata_progress_bar.total = len(tasks)
+        await self.parsing_metadata_progress_bar.asave(update_fields=["total"])
         await asyncio.gather(*tasks)
 
     async def _should_we_sync(self):
@@ -946,7 +1058,7 @@ class CollectionSyncFirstStage(Stage):
         loop = asyncio.get_event_loop()
 
         msg = _("Parsing CollectionVersion Metadata")
-        async with ProgressReport(message=msg, code="sync.parsing.metadata") as pb:
+        async with ProgressReport(message=msg, code="sync.parsing.metadata", total=0) as pb:
             self.parsing_metadata_progress_bar = pb
             await self._download_unpaginated_metadata()
 
@@ -958,6 +1070,8 @@ class CollectionSyncFirstStage(Stage):
             else:
                 tasks.append(loop.create_task(self._find_all_collections()))
             await asyncio.gather(*tasks)
+            # Ensure PR 'total' is correct before stage finishes
+            pb.total = pb.done
 
 
 class DocsBlobDownloader(ArtifactDownloader):
@@ -1024,15 +1138,17 @@ class CollectionContentSaver(ContentSaver):
         for d_content in batch:
             if d_content is None:
                 continue
-            if not isinstance(d_content.content, CollectionVersion):
-                continue
+            if isinstance(d_content.content, CollectionVersion):
+                info = d_content.content.natural_key_dict()
+                collection, created = Collection.objects.get_or_create(
+                    namespace=info["namespace"], name=info["name"]
+                )
 
-            info = d_content.content.natural_key_dict()
-            collection, created = Collection.objects.get_or_create(
-                namespace=info["namespace"], name=info["name"]
-            )
-
-            d_content.content.collection = collection
+                d_content.content.collection = collection
+            elif isinstance(d_content.content, AnsibleNamespaceMetadata):
+                name = d_content.content.name
+                namespace, created = AnsibleNamespace.objects.get_or_create(name=name)
+                d_content.content.namespace = namespace
 
     def _post_save(self, batch):
         """
