@@ -12,6 +12,7 @@ from pulp_ansible.app.galaxy.views import (
     RoleVersionList,
 )
 from pulp_ansible.app.galaxy.v3 import views as views_v3
+from pulp_ansible.app.galaxy.v3 import viewsets as viewsets_v3
 
 from pulp_ansible.app.viewsets import CopyViewSet, CollectionUploadViewSet
 
@@ -119,6 +120,24 @@ legacy_v3_collection_urls = [
         ),
         name="legacy-v3-metadata-collection-versions-list",
     ),
+    path(
+        "namespaces/",
+        views_v3.redirect_view_generator(
+            {"get": "list"},
+            url="ansible-namespaces-list",
+            viewset=views_v3.AnsibleNamespaceViewSet,
+        ),
+        name="legacy-v3-ansible-namespaces-list",
+    ),
+    path(
+        "namespaces/<str:name>/",
+        views_v3.redirect_view_generator(
+            {"get": "retrieve"},
+            url="ansible-namespaces-detail",
+            viewset=views_v3.AnsibleNamespaceViewSet,
+        ),
+        name="legacy-v3-ansible-namespaces-detail",
+    ),
     # the ansible-galaxy client doesn't play well with redirects for POST operations, so these
     # views don't redirect
     path(
@@ -155,28 +174,6 @@ v3_collection_urls = [
         views_v3.CollectionArtifactDownloadView.as_view(),
         name="collection-artifact-download",
     ),
-    path(
-        "index/<str:namespace>/<str:name>/",
-        views_v3.CollectionViewSet.as_view(
-            {"get": "retrieve", "patch": "update", "delete": "destroy"}
-        ),
-        name="collections-detail",
-    ),
-    path(
-        "index/<str:namespace>/<str:name>/versions/",
-        views_v3.CollectionVersionViewSet.as_view({"get": "list"}),
-        name="collection-versions-list",
-    ),
-    path(
-        "index/<str:namespace>/<str:name>/versions/<str:version>/",
-        views_v3.CollectionVersionViewSet.as_view({"get": "retrieve", "delete": "destroy"}),
-        name="collection-versions-detail",
-    ),
-    path(
-        "index/<str:namespace>/<str:name>/versions/<str:version>/docs-blob/",
-        views_v3.CollectionVersionDocsViewSet.as_view({"get": "retrieve"}),
-        name="collection-versions-detail-docs",
-    ),
     path(
         "all-collections/",
         views_v3.UnpaginatedCollectionViewSet.as_view({"get": "list"}),
@@ -189,10 +186,55 @@ v3_collection_urls = [
     ),
 ]
 
+v3_collection_detail_urls = [
+    path(
+        "",
+        views_v3.CollectionViewSet.as_view(
+            {"get": "retrieve", "patch": "update", "delete": "destroy"}
+        ),
+        name="collections-detail",
+    ),
+    path(
+        "versions/",
+        views_v3.CollectionVersionViewSet.as_view({"get": "list"}),
+        name="collection-versions-list",
+    ),
+    path(
+        "versions/<str:version>/",
+        views_v3.CollectionVersionViewSet.as_view({"get": "retrieve", "delete": "destroy"}),
+        name="collection-versions-detail",
+    ),
+    path(
+        "versions/<str:version>/docs-blob/",
+        views_v3.CollectionVersionDocsViewSet.as_view({"get": "retrieve"}),
+        name="collection-versions-detail-docs",
+    ),
+]
+
+namespace_urls = [
+    path(
+        "",
+        views_v3.AnsibleNamespaceViewSet.as_view({"get": "list", "post": "create"}),
+        name="ansible-namespaces-list",
+    ),
+    path(
+        "<str:name>/",
+        views_v3.AnsibleNamespaceViewSet.as_view(
+            {"get": "retrieve", "delete": "delete", "patch": "partial_update"}
+        ),
+        name="ansible-namespaces-detail",
+    ),
+]
+
 v3_plugin_urls = [
     # path:var captures /, so it has to have something at the end to make it work
     # correctly.
+    path(
+        "content/<path:distro_base_path>/collections/index/<str:namespace>/<str:name>/",
+        include(v3_collection_detail_urls),
+    ),
     path("content/<path:distro_base_path>/collections/", include(v3_collection_urls)),
+    path("content/<path:distro_base_path>/namespaces/", include(namespace_urls)),
     path(
         "imports/collections/<uuid:pk>/",
         views_v3.CollectionImportViewSet.as_view({"get": "retrieve"}),
@@ -203,6 +245,11 @@ v3_plugin_urls = [
         views_v3.ClientConfigurationView.as_view(),
         name="client-configuration-viewset",
     ),
+    path(
+        "search/collection-versions/",
+        viewsets_v3.CollectionVersionSearchViewSet.as_view({"get": "list", "post": "rebuild"}),
+        name="collection-versions-search",
+    ),
 ]
 
 v3_urls = [
