@@ -1,12 +1,13 @@
 """Tests collection sync functionality that is common to both Galaxy V2 and V3."""
 import pytest
 import subprocess
-import tempfile
 
 from os import path
 
 from pulp_ansible.tests.functional.utils import gen_ansible_remote
 
+from pulp_smash.pulp3.utils import gen_repo
+
 
 @pytest.fixture
 def sync_and_count(
@@ -32,38 +33,50 @@ def sync_and_count(
 
 
 @pytest.mark.parallel
-def test_sync_collection_from_git(ansible_repo, sync_and_count, ansible_distribution_factory):
+def test_sync_collection_from_git(
+    ansible_repo_api_client,
+    sync_and_count,
+    ansible_distribution_factory,
+    gen_user,
+    gen_object_with_cleanup,
+    ansible_dir_factory,
+    pulp_admin_user,
+):
     """Sync collections from Git repositories and then install one of them."""
-    body = gen_ansible_remote(url="https://github.com/pulp/pulp_installer.git")
+    body = gen_ansible_remote(
+        url="https://github.com/pulp/pulp_installer.git", include_pulp_auth=True
+    )
 
-    count = sync_and_count(body, repo=ansible_repo)
+    repo = gen_object_with_cleanup(ansible_repo_api_client, gen_repo())
+    count = sync_and_count(body, repo=repo)
     assert count == 1
 
     body.pop("name")  # Forces a new remote to be created
-    count = sync_and_count(body, repo=ansible_repo)
+    count = sync_and_count(body, repo=repo)
     assert count == 1
 
     body = gen_ansible_remote(url="https://github.com/pulp/squeezer.git")
 
-    count = sync_and_count(body, repo=ansible_repo)
+    count = sync_and_count(body, repo=repo)
     assert count == 2
 
     # Create a distribution.
-    distribution = ansible_distribution_factory(ansible_repo)
+    distribution = ansible_distribution_factory(repo)
 
     collection_name = "pulp.squeezer"
-    with tempfile.TemporaryDirectory() as temp_dir:
 
+    temp_dir = ansible_dir_factory(distribution.client_url, pulp_admin_user)
+    with temp_dir:
         # The install command needs --pre so a pre-release collection versions install
-        cmd = "ansible-galaxy collection install --pre {} -c -s {} -p {}".format(
-            collection_name, distribution.client_url, temp_dir
+        cmd = "ansible-galaxy collection install --pre {} -c -p {}".format(
+            collection_name, temp_dir
         )
 
         directory = "{}/ansible_collections/{}".format(temp_dir, collection_name.replace(".", "/"))
 
         assert not path.exists(directory), "Directory {} already exists".format(directory)
 
-        subprocess.run(cmd.split())
+        subprocess.run(cmd.split(), cwd=temp_dir)
 
         assert path.exists(directory), "Could not find directory {}".format(directory)
 
@@ -150,6 +163,7 @@ def test_sync_metadata_only_collection_from_pulp(
         url="https://github.com/ansible-collections/amazon.aws/",
         metadata_only=True,
         git_ref="2.1.0",
+        include_pulp_auth=True,
     )
     amazon_remote = ansible_git_remote_factory(**body)
 
